package pps.core.prediction.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.auth.HttpRequestClient;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.ConfigShowEnt;
import pps.core.prediction.entity.PlantPredictedPowerMidTermEnt;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.mapper.ConfigShowMapper;
import pps.core.prediction.mapper.PlantPredictedPowerMidTermMapper;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPredictedPowerOutput;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XHttpUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2024/03/18 15:07
 */
public class ServiceUtil {

    /**
     * 调用接口(长庆)
     *
     * @param context 上下文
     * @param api     应用程序编程接口
     * @param param   参数
     * @return {@link String}
     */
    public static String doPostFormCq(XContext context, String api, Map<String, Object> param) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        String resultStr = client.doPostForm(url + api, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        context.getLogger().info("**** doPostFormCq api: {}", api);
        context.getLogger().info("**** doPostFormCq resultStr: {}", resultStr);
        return jsonObject.getString("data");
    }

    /**
     * 获取长庆TOKEN
     *
     * @param context 上下文
     * @return {@link String}
     */
    public static String getToken(XContext context) {
        String token;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            token = ServiceUtil.getCqToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                token = ServiceUtil.getCqToken(context);
            } else {
                token = exist.getCode();
            }
        }
        return token;
    }

    /**
     * 获得预测功率输出
     *
     * @param context      上下文
     * @param inputTimeStr 输入时间str
     * @param stationId    工作站id
     * @param offsetDay    偏移日
     * @param cycle        周期
     * @return {@link List }<{@link GetPredictedPowerOutput }>
     */
    public static List<GetPredictedPowerOutput> getPredictedPowerOutput(XContext context, String inputTimeStr, String stationId, int offsetDay, int cycle) {
        //判断是否需要启用演示配置
        DateTime date = DateUtil.date();
        DateTime inputTime = DateUtil.parse(inputTimeStr, BusinessConstant.DATE_FORMAT_DAY);
        ConfigShowMapper configMapper = context.getBean(ConfigShowMapper.class);
        ConfigShowEnt configShowEnt = configMapper.selectOne(new LambdaQueryWrapper<ConfigShowEnt>()
                .last("LIMIT 1"));
        Date showTime;
        DateTime day;
        int betweenDay;
        if (ObjectUtil.isAllNotEmpty(configShowEnt, configShowEnt.getIsEnable(), configShowEnt.getChoiceDate())) {
            day = configShowEnt.getIsEnable().equals(0) ? DateUtil.date(configShowEnt.getChoiceDate()) : inputTime;
            showTime = DateUtil.parse(day.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + date.toString(BusinessConstant.TIME_FORMAT));
            //计算偏移
            betweenDay = (int) DateUtil.betweenDay(inputTime, day, false);
            if (DateUtil.compare(day, inputTime) > 0) {
                betweenDay = -betweenDay;
            }
        } else {
            day = inputTime;
            showTime = date;
            betweenDay = 0;
        }
        DateTime beginTime = DateUtil.offsetMinute(day, -1);
        DateTime endTime = DateUtil.offsetDay(day, offsetDay);
        //查预测功率
        Map<DateTime, BigDecimal> predictedMap;
        switch (cycle) {
            case 1:
                //查短期预测
                PlantPredictedPowerShortTermMapper shortMapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
                List<PlantPredictedPowerShortTermEnt> predictedList = shortMapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerShortTermEnt>()
                        .eq(PlantPredictedPowerShortTermEnt::getPlantId, stationId)
                        .between(PlantPredictedPowerShortTermEnt::getDataDate, beginTime, endTime)
                        .orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(predictedList)) {
                    predictedMap = predictedList.stream()
                            .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerShortTermEnt::getPredictPower));
                } else {
                    predictedMap = Collections.emptyMap();
                }
                break;
            case 2:
                //查中期预测
                PlantPredictedPowerMidTermMapper midMapper = context.getBean(PlantPredictedPowerMidTermMapper.class);
                List<PlantPredictedPowerMidTermEnt> midList = midMapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerMidTermEnt>()
                        .eq(PlantPredictedPowerMidTermEnt::getPlantId, stationId)
                        .between(PlantPredictedPowerMidTermEnt::getDataDate, beginTime, endTime)
                        .orderByAsc(PlantPredictedPowerMidTermEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(midList)) {
                    predictedMap = midList.stream()
                            .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerMidTermEnt::getPredictPower));
                } else {
                    predictedMap = Collections.emptyMap();
                }
                break;
            default:
                predictedMap = Collections.emptyMap();
        }
        //查实际功率
        ThirdActivePowerViewMapper powerMapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> powerList = powerMapper.selectAvgPower15(ThirdActivePowerView.builder()
                .stationId(stationId)
                .startTime(beginTime)
                .endTime(showTime)
                .build());
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(powerList)) {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getInputTime, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        //取时间范围
        List<DateTime> rangeToList = DateUtil.rangeToList(day, endTime, DateField.MINUTE, 15);
        GetPredictedPowerOutput output;
        List<GetPredictedPowerOutput> outputs = new ArrayList<>(rangeToList.size());
        for (DateTime dateTime : rangeToList) {
            output = new GetPredictedPowerOutput();
            //匹配预测发电量
            output.setPredictPower(predictedMap.getOrDefault(dateTime, BigDecimal.ZERO));
            //匹配实际发电量
            output.setPower(powerMap.getOrDefault(dateTime, BigDecimal.ZERO));
            //时间偏移
            if (betweenDay != 0) {
                output.setDataDate(DateUtil.offsetDay(dateTime, betweenDay));
            } else {
                output.setDataDate(dateTime);
            }
            outputs.add(output);
        }
        return outputs;
    }

    /*----------------------------------- 私有方法 -----------------------------------*/

    /**
     * 获取长庆Token
     *
     * @return {@link String}
     */
    private static String getCqToken(XContext context) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_TOKEN;
        Map<String, String> param = new HashMap<>(4);
        param.put("UserAccount", "PvGroup");
        param.put("UserPassWord", "PvGroup@147369");
        String result = XHttpUtils.postAsForm(url, param);
        JSONObject jsonObject = JSON.parseObject(result);
        String token = jsonObject.getString("token");
        if (CharSequenceUtil.isBlank(token)) {
            throw new XServiceException("获取长庆Token失败");
        }
        ServiceUtil.setThirdPartyConfigCache(context, token);
        return token;
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param token   代币
     */
    private static void setThirdPartyConfigCache(XContext context, String token) {
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        cache.setCode(token);
        cache.setValidity(90);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
    }
}
